Contributing
============

Quick summary:

 - Use [topical branches][q1] and standard PR etiquette.
 - You need to formally [agree to release][q2] your contribution.
 - Both [linting and testing][q3] should pass.
 - There are various ways to [get help][q4].

[q1]: #topical-branches
[q2]: #copyright--release
[q3]: #style--tests
[q4]: #support


Standard Etiquette
------------------

### Introduction

Most git projects nowadays use a variation on the same branching
and workflow model. Here's [a great writeup][se1] of quite a rigid
implementation of the model. Simpler projects often do away with
the fluff and focus on the essential.

### Main branch

There's only one main branch: __master__. The rest are features
and fixes. The __gh-pages__ branch is separate, and has generated
documentation; you don't need to touch it.

### Topical branches

The rest of the branches are for features and fixes. They should
have descriptive names. This is no only to be pretty, but also so
a maintainer can pull a contributor's branch and test it locally
without clobbering any work.

Use the present tense and try to keep it short. There's no prefixes
defined to differentiate features and fixes, but you can use these
if you wish to. Branch names should be `[a-z0-9\-]`: lowercase
alphanumerics with hyphens.

For example, a feature branch to implement stream pooling could be
named `implement-stream-pooling` or `stream-pooling` or `stream-pools`
or `feature-stream-pooling`. A bugfix branch to repair tests failing
due to timeouts being too low could be named `fix-tests-timeouts` or
`longer-tests-timeouts`.

### Version tags

Tags should only be used for versions, which follow the `vN.N.N` model.
Tags should correspond with the code as is available in [npm][se2] for
that release. They are set by maintainers only. They follow [semver][se5].

### Commits

Always rebase your commit history to present a clean face. When asked
to fix typos or do small modifications, fold them in by amending the
previous commit and forcing a push. Don't mention the file you have
edited in the commit message if it's only a single file: it's redundant
information. For example, don't use `Update README.md` but rather
`Fix typo in code sample`.

Commit messages should be [well formed][se3] (if you're reading that
blog post, ignore the double spaces after periods; ugly shit like that
will be rectified pronto). If you have used code from somebody else, or
inspired by somebody else, mention them in the commit. Include links if
necessary. The last paragraph should be a newline-delimited list of
pertinent issues, if applicable.

For example:

```
Capitalized, short (50 chars or less) summary

More detailed explanatory text, if necessary. Wrap it to about 72-80
characters or so. In some contexts, the first line is treated as the
subject of an email and the rest of the text as the body. The blank
line separating the summary from the body is critical (unless you omit
the body entirely); tools like rebase can get confused if you run the
two together.

Write your commit message in the present tense: "Fix bug" and not "Fixed
bug." This convention matches up with commit messages generated by
commands like git merge and git revert.

Further paragraphs come after blank lines.

 - Bullet points are okay, too

 - Typically a hyphen or asterisk is used for the bullet, preceded by a
   single space, with blank lines in between, but conventions vary here

 - Use a hanging indent

Array folding snippet in examplefile.js inspired by foobar's solution
to http://stackoverflow.com/q/123456789/array-folding-in-javascript

Fixes #123
See #45 and #67 for details
Opens the door to work on #89
```

### Pull requests

Standard internet etiquette (spelling, grammar, be courteous). Also:

 - Include a good description of the changes, except if the title
   is self-evident.

 - If you're touching on things discussed in other issues, mention
   them. If you're committing code that's not yours, mention that.
   If there's any other resources you think are relevant, include
   them (using a link).

 - If there's things remaining to be done, like tests or documentation,
   or even details to discuss, include [a checklist][se4]. Tick it off
   as stuff is done.

[se1]: http://nvie.com/posts/a-successful-git-branching-model/
[se2]: https://npmjs.org/package/mtwitter
[se3]: http://tbaggery.com/2008/04/19/a-note-about-git-commit-messages.html
[se4]: https://github.com/blog/1375-task-lists-in-gfm-issues-pulls-comments
[se5]: http://semver.org

Copyright & Release
-------------------

__mtwitter__ is released in the Public Domain (or licensed under [CC0][c1]
if applicable). To keep the project free of legal trouble, you need to
include this statement (inspired by the [CC0 waiver][c2]) in your first
pull request:

> I hereby waive all copyright and related or neighboring rights together
> with all associated claims and causes of action with respect to this
> work to the extent possible under the law. This waiver shall be kept in
> effect for all future work I might contribute to this project until I
> give written notice of the contrary.

__ntwitter__, the project this was forked from, was licensed under an MIT
license. When I unlicensed __mtwitter__, it had been extensively reworked
and beared little enough resemblance (about 5% similarity) that I considered
I arguably had written the work and therefore held the copyright.

If you disagree, you are welcome to discuss it and scold me were I to be
proved wrong.

[c1]: https://creativecommons.org/publicdomain/zero/1.0
[c2]: https://creativecommons.org/choose/zero/waiver


Style & Tests
-------------

All pull requests and pushes are automatically linted by Travis CI.
If the linting fails, the build fails, and you'll get a red cross.
Linting is generally easy to fix. Run `npm run-script lint` to make
sure everything is ok.

Additionally, there are a few things listed in the [style guide][s1]
which aren't tested by the lint check. Maintainers will look for these
manually when reviewing the pull request.

You should write tests if you can. If you can't, or aren't sure, or
want immediate feedback, submit the pull request and include "Write
tests" in the task list. Due to the way Travis CI works, the test
suite isn't run for pull requests. Testing is done manually by the
maintainers.

You can run the test suite by using `npm test`.

[s1]: https://github.com/passcod/node-style-guide


Support
-------

You can reach out (in order of preference):

 - By opening or commenting on an issue.
 - By asking on __#mtwitter__ on [Freenode][s2].
 - By sending me an email: me@passcod.name (be clear and patient).

I can only spend so much time on this per week, so do not expect
things, bla bla bla, insert standard disclaimer.

[s2]: https://freenode.net
